#!/usr/bin/python3

from aws_requests_auth.aws_auth import AWSRequestsAuth
from xml.dom import minidom
import argparse
import requests
import pathlib
import uuid
import time
from typing import List

parser = argparse.ArgumentParser(description='Invalidate game files on the amazon CDN.')
parser.add_argument('games', metavar='name', nargs='*',
                    help='List of game folders to invalidate (use \'core\' to invalidate the core pack, \'checksummer\' to update the checksummer)')
parser.add_argument('--key', default="", help='The AWS access key')
parser.add_argument('--secret', default="", help='The AWS secret key')
parser.add_argument('--jurisdiction', default="DEFAULT", help='The jurisdiction to invalidate')
parser.add_argument('--version', default="V7.1", help='The version to invalidate')
parser.add_argument('--show-error-requests', action="store_true",
                    help='If given, invalidations which trigger an error will have their original request printed out')
parser.add_argument('--wildcard-fallback', action="store_true",
                    help='If given, invalidations that fail will bne retried with a wildcard path')

args = parser.parse_args()

CDN1_Webgames_Dist_ID = "E3I86RA8WCOTN2"  # domain name https://dv8rz1583o82y.cloudfront.net
CDN2_Packages_Dist_ID = "EOFB63RBYMVAC"  # domain name http://dubl7696qavg.cloudfront.net
API_PROTOCOL = "https"
API_VERSION = "2020-05-31"
AWS_API_HOST = "cloudfront.amazonaws.com"
REGION = "us-east-1"  # cloudfront has no region, but requires us to use this default
INNER_GAME_PACKAGES_PATH = "/opt/imaxa/package_source/GAMES_v7/Inner/"

auth = AWSRequestsAuth(aws_access_key=args.key,
                       aws_secret_access_key=args.secret,
                       aws_host=AWS_API_HOST,
                       aws_service='cloudfront',
                       aws_region=REGION)


def create_invalidation_body(paths: List[str]) -> str:
    root = minidom.Document()
    batch = root.createElement("InvalidationBatch")
    batch.setAttribute("xmlns", "http://%s/doc/%s/" % (AWS_API_HOST, API_VERSION))
    callerRef = root.createElement("CallerReference")
    callerRef.appendChild(root.createTextNode(uuid.uuid4().hex))
    batch.appendChild(callerRef)
    pathsXml = root.createElement("Paths")
    pathItems = root.createElement("Items")
    for path in paths:
        thisPathXml = root.createElement("Path")
        thisPathXml.appendChild(root.createTextNode(path))
        pathItems.appendChild(thisPathXml)
    pathsXml.appendChild(pathItems)
    quantity = root.createElement("Quantity")
    quantity.appendChild(root.createTextNode(str(len(paths))))
    pathsXml.appendChild(quantity)
    batch.appendChild(pathsXml)
    root.appendChild(batch)
    return root.toprettyxml()


def invalidate(distribution: str, paths: List[str], desc: str) -> minidom.Document:
    if len(paths) == 0:
        return

    request_body = create_invalidation_body(paths)
    res = requests.post("%s://%s/%s/distribution/%s/invalidation" % (API_PROTOCOL, AWS_API_HOST, API_VERSION, distribution),
                        request_body, auth=auth, headers={"Content-Type": "application/xml", "User-Agent": "The Meep Man"})
    response_body = res.content.decode()
    if res.status_code == 201:
        print("Successfully invalidated %d files on distribution %s (%s)" % (len(paths), distribution, desc))
    else:
        print("! Error during invalidation of %d files on distribution %s (%s)" % (len(paths), distribution, desc))
        print("! status %d" % (res.status_code))
        if args.show_error_requests:
            print("! request body %s" % (request_body))
        print("! response body %s" % (response_body))
    return minidom.parseString(response_body)

def getText(nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    return ''.join(rc)

# first invalidate the webgames
for game in args.games:
    if game == "core":
        continue
    invalidation_batch = []
    gameContent = pathlib.Path(INNER_GAME_PACKAGES_PATH + game)
    for item in gameContent.rglob("*"):
        if item.is_file():
            invalidation_batch.append("/" + str(item.relative_to(INNER_GAME_PACKAGES_PATH)))
    response = invalidate(CDN1_Webgames_Dist_ID, invalidation_batch, game)
    errors = response.getElementsByTagName("ErrorResponse")
    if (args.wildcard_fallback and errors.length == 1 and getText(errors[0].getElementsByTagName("Code")[0].childNodes) == "InvalidArgument"):
        print("! retrying with wildcard path...")
        time.sleep(2.0)
        invalidate(CDN1_Webgames_Dist_ID, ["/" + game + "/*"], game)

    time.sleep(2.0)

# then invalidate the packages
package_invalidation_list = []
for game in args.games:
    files = []
    if game == "core":
        files.append("core_gameart.pack.md5")
        files.append("core_gameart.pack")
    elif game == "checksummer":
        files.append("checksummer.pack.md5")
        files.append("checksummer.pack")
    else:
        files.append("Games/" + game + ".pack.md5")
        files.append("Games/" + game + ".info")
        files.append("Games/" + game + ".pack")

    for file in files:
        package_invalidation_list.append("/PACKAGES_IGP/%s/%s/%s" % (args.jurisdiction, args.version, file))

invalidate(CDN2_Packages_Dist_ID, package_invalidation_list, "packages")

